@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 # Copyright (C) 2016 Red Hat, Inc.
 # This file is part of libsan.
 #
@@ -21,57 +20,51 @@ __author__ = "Bruno Goncalves"
 __copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."
 
 import re  # regex
-import augeas
-import libsan.host.mp
+
 import libsan.host.linux
-import libsan.host.scsi
+import libsan.host.mp
 import libsan.host.net
+import libsan.host.scsi
+from libsan import _print
 from libsan.host.cmdline import run
 
 # used to match regex for each session information that we support
-supported_discovery_info = {"address": r".*DiscoveryAddress: (\S+)",
-                            "target": r".*Target: (\S+)",
-                            "portal": r".*Portal: (\S+):(\S+),(\S+)",
-                            "iface": r".*Iface Name: (\S+)"}
+supported_discovery_info = {
+    "address": r".*DiscoveryAddress: (\S+)",
+    "target": r".*Target: (\S+)",
+    "portal": r".*Portal: (\S+):(\S+),(\S+)",
+    "iface": r".*Iface Name: (\S+)",
+}
 
 # used to match regex for each session information that we support
-supported_session_info = {"t_iqn": r".*Target: (\S+)",
-                          "h_iqn": r".*Iface Initiatorname: (\S+)",
-                          "iface": r".*Iface Name: (\S+)",
-                          "transport": r".*Iface Transport: (\S+)",
-                          "iface_ip": r".*Iface IPaddress: (\S+)",
-                          "mac": r".*Iface HWaddress: (\S+)",
-                          "sid": r".*SID: (\S+)",
-                          "host": r".*Host Number: (\S+).*State: (\S+)",  # eg. Host Number: 6	State: running
-                          "disks": r".*Attached scsi disk (\S+).*State: (\S+)",
-                          # eg. Attached scsi disk sdb		State: running
-                          "target_ip": r".*Current Portal: (\S+):[0-9]+,",
-                          "persist_ip": r".*Persistent Portal: (\S+):[0-9]+,",
-                          # negotiated parameters
-                          "header_digest": r".*HeaderDigest: (\S+)",
-                          "data_digest": r".*DataDigest: (\S+)",
-                          "max_recv": r".*MaxRecvDataSegmentLength: (\S+)",
-                          "max_xmit": r".*MaxXmitDataSegmentLength: (\S+)",
-                          "first_burst": r".*FirstBurstLength: (\S+)",
-                          "max_burst": r".*MaxBurstLength: (\S+)",
-                          "immediate_data": r".*ImmediateData: (\S+)",
-                          "initial_r2t": r".*InitialR2T: (\S+)",
-                          "max_outst_r2t": r".*MaxOutstandingR2T: (\S+)"}
+supported_session_info = {
+    "t_iqn": r".*Target: (\S+)",
+    "h_iqn": r".*Iface Initiatorname: (\S+)",
+    "iface": r".*Iface Name: (\S+)",
+    "transport": r".*Iface Transport: (\S+)",
+    "iface_ip": r".*Iface IPaddress: (\S+)",
+    "mac": r".*Iface HWaddress: (\S+)",
+    "sid": r".*SID: (\S+)",
+    "host": r".*Host Number: (\S+).*State: (\S+)",  # e.g. Host Number: 6	State: running
+    "disks": r".*Attached scsi disk (\S+).*State: (\S+)",
+    # eg. Attached scsi disk sdb		State: running
+    "target_ip": r".*Current Portal: (\S+):[0-9]+,",
+    "persist_ip": r".*Persistent Portal: (\S+):[0-9]+,",
+    # negotiated parameters
+    "header_digest": r".*HeaderDigest: (\S+)",
+    "data_digest": r".*DataDigest: (\S+)",
+    "max_recv": r".*MaxRecvDataSegmentLength: (\S+)",
+    "max_xmit": r".*MaxXmitDataSegmentLength: (\S+)",
+    "first_burst": r".*FirstBurstLength: (\S+)",
+    "max_burst": r".*MaxBurstLength: (\S+)",
+    "immediate_data": r".*ImmediateData: (\S+)",
+    "initial_r2t": r".*InitialR2T: (\S+)",
+    "max_outst_r2t": r".*MaxOutstandingR2T: (\S+)",
+}
 
 host_path = "/sys/class/iscsi_host/"
 
 
-def _print(string):
-    module_name = __name__
-    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
-    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
-    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
-    print(string)
-    if "FATAL:" in string:
-        raise RuntimeError(string)
-    return
-
-
 def is_iqn(iqn):
     if re.match(r"^iqn\.", iqn):
         return True
@@ -134,13 +127,12 @@ def query_discovery():
         # print "(%s)" % line
         # Check if it is discovery mode information
         m = re.match("(^.*):", line)
-        if m:
-            if m.group(1) in supported_discovery_modes:
-                disc_mode = m.group(1)
-                # We will use DiscoveryAddress as key
-                discovery_info_dict[disc_mode] = {}
-                discovery_address = None
-                continue
+        if m and m.group(1) in supported_discovery_modes:
+            disc_mode = m.group(1)
+            # We will use DiscoveryAddress as key
+            discovery_info_dict[disc_mode] = {}
+            discovery_address = None
+            continue
 
         # We will use TargetAddress as key for the target dictionary
         m = re.match(supported_discovery_info["address"], line)
@@ -195,7 +187,7 @@ def discovery_st(target, ifaces=None, disc_db=False):
     The arguments are:
     \ttarget:   Address of target to be discovered
     \tifaces:   iSCSI interfaces to be used, separated by space (optional)
-    \tdisc_db:  if should use discoverydb instead of discovery (optional)
+    \tdisc_db:  To use discoverydb instead of discovery (optional)
     Returns:
     \tTrue:     If it discovered an iSCSI target
     \tFalse:    If some problem happened
@@ -212,11 +204,13 @@ def discovery_st(target, ifaces=None, disc_db=False):
         disc_opt = "discoverydb -D"
         operation = "new"
 
-    cmd = "iscsiadm -m %s -p %s" % (disc_opt, target)
+    cmd = f"iscsiadm -m {disc_opt} -p {target}"
     if operation:
         cmd += " -o %s" % operation
 
     if ifaces:
+        if ("bnx2i" in ifaces or "qedi" in ifaces) and libsan.host.linux.service_status("iscsiuio", verbose=False) != 0:
+            libsan.host.linux.service_enable("iscsiuio", now=True)
         interfaces = ifaces.split(" ")
         for interface in interfaces:
             cmd += " -I %s" % interface
@@ -258,8 +252,7 @@ def is_target_discovered(t_iqn):
 
 
 def get_disc_ifaces_of_t_iqn(t_iqn):
-    """
-    From given target IQN, return the interfaces that discovered it
+    """From given target IQN, return the interfaces that discovered it
     The arguments are:
     \tiSCSI Target:   iQN of iSCSI target
     Returns:
@@ -280,7 +273,7 @@ def get_disc_ifaces_of_t_iqn(t_iqn):
         for disc_addr in list(disc_dict[disc_type].keys()):
             if "targets" not in list(disc_dict[disc_type][disc_addr].keys()):
                 continue
-            if t_iqn in list(disc_dict[disc_type][disc_addr]["targets"].keys()):
+            if t_iqn in list(disc_dict[disc_type][disc_addr]["targets"].keys()):  # noqa: SIM102
                 if "iface" in list(disc_dict[disc_type][disc_addr]["targets"][t_iqn].keys()):
                     return disc_dict[disc_type][disc_addr]["targets"][t_iqn]["iface"]
     return None
@@ -301,7 +294,7 @@ def delete_discovery_target_portal(portal, port="3260", tp="st"):
         # IF IPv6 we need to append "[" and "]" to the address
         portal = "[" + portal + "]"
 
-    cmd = "iscsiadm -m discoverydb --type %s --portal \"%s:%s\" -o delete" % (tp, portal, port)
+    cmd = f'iscsiadm -m discoverydb --type {tp} --portal "{portal}:{port}" -o delete'
     retcode, _ = run(cmd, return_output=True, verbose=True)
     if retcode != 0:
         _print("FAIL: Could not delete discover iSCSI target")
@@ -356,7 +349,7 @@ def clean_up(portal="all"):
     # Not iSCSI boot
     if portal == "all":
         # log out of all iSCSI sessions
-        if get_all_session_ids():
+        if get_all_session_ids():  # noqa: SIM102
             # There is at least one session
             if not node_logout():
                 _print("FAIL: Could not logout from %s iSCSI target" % portal)
@@ -381,7 +374,7 @@ def clean_up(portal="all"):
 
                 disc_addr = d_dict["disc_addr"]
                 port = d_dict["disc_port"]
-                if disc_addr == portal or portal == "all":
+                if disc_addr == portal or portal == "all":  # noqa: SIM102
                     if not delete_discovery_target_portal(disc_addr, port=port, tp=d_dict["mode"]):
                         _print("FAIL: Deleting iSCSI target %s" % d_dict["disc_addr"])
                         error += 1
@@ -413,6 +406,7 @@ def clean_up(portal="all"):
 #            sessions[sid] = ses_dict
 #    return sessions_dict
 
+
 def get_all_session_ids():
     cmd = "iscsiadm -m session -P1"
     retcode, output = run(cmd, return_output=True, verbose=False)
@@ -433,8 +427,7 @@ def get_all_session_ids():
 
 
 def query_iscsi_session(sid):
-    """
-    Query information from an specific iSCSI session
+    """Query information from a specific iSCSI session
     The arguments are:
     \tsid:      Session ID
     Returns:
@@ -467,7 +460,7 @@ def query_iscsi_session(sid):
             bus_id_only = m.group(3)
             lun_id = m.group(4)
             target_id_only = re.sub("^0+(?=.)", "", target_id_only)
-            scsi_id = "%s:%s:%s:%s" % (host_id, target_id_only, bus_id_only, lun_id)
+            scsi_id = f"{host_id}:{target_id_only}:{bus_id_only}:{lun_id}"
 
             if "scsi_id_info" not in list(session_info_dict.keys()):
                 session_info_dict["scsi_id_info"] = {}
@@ -477,7 +470,7 @@ def query_iscsi_session(sid):
         # Could be more than one scsi disk, will add as dict
         m = re.match(supported_session_info["disks"], line)
         if m:
-            disk_dict = dict()
+            disk_dict = {}
             # disk_dict["scsi_name"] = m.group(1)
             disk_dict["status"] = m.group(2)
             disk_dict["wwid"] = libsan.host.scsi.wwid_of_disk(m.group(1))
@@ -498,7 +491,7 @@ def query_iscsi_session(sid):
             session_info_dict[key] = m.group(1)
             if session_info_dict[key] == "<empty>":
                 session_info_dict[key] = None
-                if key == "mac":
+                if key == "mac":  # noqa: SIM102
                     # Try to get based on iface IP address
                     if "iface_ip" in list(session_info_dict.keys()):
                         nic = libsan.host.net.get_nic_of_ip(session_info_dict["iface_ip"])
@@ -511,9 +504,7 @@ def query_iscsi_session(sid):
 
 
 def query_all_iscsi_sessions():
-    """
-    First we get all iSCSI ids, later on we get the information of each session individually
-    """
+    """First we get all iSCSI ids, later on we get the information of each session individually"""
 
     session_ids = get_all_session_ids()
     if not session_ids:
@@ -530,8 +521,6 @@ def query_all_iscsi_sessions():
 
 
 def session_logout(sid=None):
-    """
-    """
     cmd = "iscsiadm -m session -u"
     if sid:
         cmd += " -r %s" % sid
@@ -544,9 +533,7 @@ def session_logout(sid=None):
 
 
 def get_iscsi_session_by_scsi_id(scsi_id):
-    """
-    Return the Session Dict that has the scsi_id
-    """
+    """Return the Session Dict that has the scsi_id"""
     sessions = query_all_iscsi_sessions()
     if not sessions:
         return None
@@ -560,8 +547,7 @@ def get_iscsi_session_by_scsi_id(scsi_id):
 
 
 def h_iqn_of_sessions():
-    """
-    Usage
+    """Usage
         h_iqn_of_sessions()
     Purpose
         Get the Host IQNs of all active iSCSI sessions
@@ -588,8 +574,7 @@ def h_iqn_of_sessions():
 
 
 def t_iqn_of_sessions():
-    """
-    Usage
+    """Usage
         t_iqn_of_sessions()
     Purpose
         Get the Target IQNs of all active iSCSI sessions
@@ -616,8 +601,7 @@ def t_iqn_of_sessions():
 
 
 def mac_of_iscsi_session():
-    """
-    Usage
+    """Usage
         mac_of_iscsi_session()
     Purpose
         We only check host IQN in active iSCSI session.
@@ -644,8 +628,7 @@ def mac_of_iscsi_session():
 
 
 def scsi_names_of_iscsi_session(h_iqn=None, t_iqn=None, sid=None):
-    """
-    Usage
+    """Usage
         scsi_names_of_iscsi_session();
         scsi_names_of_iscsi_session(sid=1);
         scsi_names_of_iscsi_session(h_iqn=h_iqn, t_iqn=t_iqn);
@@ -670,9 +653,8 @@ def scsi_names_of_iscsi_session(h_iqn=None, t_iqn=None, sid=None):
         return None
 
     if sid:
-        if sid in list(sessions.keys()):
-            if "disks" in list(sessions[sid].keys()):
-                return list(sessions[sid]["disks"].keys())
+        if sid in list(sessions.keys()) and "disks" in list(sessions[sid].keys()):
+            return list(sessions[sid]["disks"].keys())
         return None
 
     scsi_names = None
@@ -686,8 +668,7 @@ def scsi_names_of_iscsi_session(h_iqn=None, t_iqn=None, sid=None):
 
     if h_iqn and t_iqn:
         for sid in list(sessions.keys()):
-            if (sessions[sid]["h_iqn"] == h_iqn and
-                    sessions[sid]["t_iqn"] == t_iqn):
+            if sessions[sid]["h_iqn"] == h_iqn and sessions[sid]["t_iqn"] == t_iqn:  # noqa: SIM102
                 if "disks" in list(sessions[sid].keys()):
                     if not scsi_names:
                         scsi_names = []
@@ -699,8 +680,7 @@ def scsi_names_of_iscsi_session(h_iqn=None, t_iqn=None, sid=None):
 
 
 def scsi_wwid_of_iscsi_session(h_iqn=None, t_iqn=None, sid=None):
-    """
-    Usage
+    """Usage
         scsi_wwid_of_iscsi_session();
         scsi_wwid_of_iscsi_session(sid=1);
         scsi_wwid_of_iscsi_session(h_iqn=h_iqn, t_iqn=t_iqn);
@@ -753,8 +733,7 @@ def scsi_wwid_of_iscsi_session(h_iqn=None, t_iqn=None, sid=None):
 
     if h_iqn and t_iqn:
         for sid in list(sessions.keys()):
-            if (sessions[sid]["h_iqn"] == h_iqn and
-                    sessions[sid]["t_iqn"] == t_iqn):
+            if sessions[sid]["h_iqn"] == h_iqn and sessions[sid]["t_iqn"] == t_iqn:  # noqa: SIM102
                 if "disks" in list(sessions[sid].keys()):
                     if not wwids:
                         wwids = []
@@ -769,8 +748,6 @@ def scsi_wwid_of_iscsi_session(h_iqn=None, t_iqn=None, sid=None):
 
 
 def is_iscsi_boot():
-    """
-    """
     iscsi_wwids = scsi_wwid_of_iscsi_session()
     if not iscsi_wwids:
         return False
@@ -808,8 +785,8 @@ def node_login(options=None, target=None, portal=None, udev_wait_time=15):
         cmd += " %s" % options
 
     if target:
-        for target in target.split():
-            cmd += " -T %s" % target
+        for target_iqn in target.split():
+            cmd += " -T %s" % target_iqn
 
     if portal:
         cmd += " -p %s" % portal
@@ -857,9 +834,7 @@ def node_logout(options=None, target=None, portal=None):
 
 
 def node_delete(options=None):
-    """
-    Delete node information
-    """
+    """Delete node information"""
     if not options:
         _print("FAIL: node_delete() - requires portal and/or target parameters")
         return False
@@ -878,14 +853,10 @@ def node_delete(options=None):
 
 # iSCSI iface ###
 def iface_query_all_info(iface_name=None):
-    """
-    Return a dict with interface names as key with detailed information of
+    """Return a dict with interface names as key with detailed information of
     interface
     """
-    if iface_name:
-        ifaces = [iface_name]
-    else:
-        ifaces = get_iscsi_iface_names()
+    ifaces = [iface_name] if iface_name else get_iscsi_iface_names()
 
     if not ifaces:
         return None
@@ -933,19 +904,18 @@ def iface_update(iface, name, value):
         _print("FAIL: iface_update() - required parameters: iface, name, value")
         return False
 
-    cmd = "iscsiadm -m iface -I %s -o update -n iface.%s -v %s" % (iface, name, value)
+    cmd = f"iscsiadm -m iface -I {iface} -o update -n iface.{name} -v {value}"
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
-        _print("FAIL: Could not set %s to %s on iface %s" % (name, value, iface))
+        _print(f"FAIL: Could not set {name} to {value} on iface {iface}")
         print(output)
         return False
 
     return True
 
 
-def iface_set_iqn(iqn, iface='default'):
-    """
-    Set IQN in /etc/iscsi/initiatorname or for specific iface
+def iface_set_iqn(iqn, iface="default"):
+    """Set IQN in /etc/iscsi/initiatorname or for specific iface
     Return:
         True
         of
@@ -955,9 +925,9 @@ def iface_set_iqn(iqn, iface='default'):
         _print("FAIL: iface_set_iqn() - requires iqn to be set")
         return False
 
-    if iface == 'default':
+    if iface == "default":
         try:
-            with open("/etc/iscsi/initiatorname.iscsi", "w", encoding='UTF-8') as i:
+            with open("/etc/iscsi/initiatorname.iscsi", "w", encoding="UTF-8") as i:
                 i.write("InitiatorName=" + str(iqn))
         except Exception as e:
             _print("FAIL: Could not set iqn in /etc/iscsi/initiatorname.iscsi " + e.__str__())
@@ -966,15 +936,14 @@ def iface_set_iqn(iqn, iface='default'):
 
         return True
 
-    if not iface_update(iface, 'initiatorname', iqn):
+    if not iface_update(iface, "initiatorname", iqn):
         return False
 
     return True
 
 
 def iface_set_ip(iface, ip, mask=None, gw=None):
-    """
-    Set IP information for specific iface
+    """Set IP information for specific iface
     Return:
         True
         of
@@ -984,69 +953,45 @@ def iface_set_ip(iface, ip, mask=None, gw=None):
         _print("FAIL: iface_set_ip() - requires iface and ip parameters")
         return False
 
-    if not iface_update(iface, 'ipaddress', ip):
+    if not iface_update(iface, "ipaddress", ip):
         return False
 
-    if mask:
-        if not iface_update(iface, 'subnet_mask', mask):
-            return False
+    if mask and not iface_update(iface, "subnet_mask", mask):
+        return False
 
-    if gw:
-        if not iface_update(iface, 'gateway', gw):
-            return False
+    if gw and not iface_update(iface, "gateway", gw):
+        return False
 
     return True
 
 
 def get_iscsi_iface_names():
-    """
-    Return a list with the name of all iSCSI interfaces on the host
-    """
-    cmd = "iscsiadm -m iface | cut -d \" \" -f 1"
+    """Return a list with the name of all iSCSI interfaces on the host"""
+    cmd = 'iscsiadm -m iface | cut -d " " -f 1'
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
         _print("FAIL: Could not read iSCSI interfaces")
         print(output)
         return None
     ifaces = output.split("\n")
+    ifaces[:] = (value for value in ifaces if "iSCSI ERROR" not in value)  # bz1997710
     return ifaces
 
 
-def set_iscsid_parameter(**kwargs):
-    """
-    Change parameter in iscsid.conf file and restarts iscsid service
+def set_iscsid_parameter(parameters):
+    """Change parameter in iscsid.conf file and restarts iscsid service
     Use dictionary with parameter:value as argument
     """
 
-    aug = augeas.Augeas()
-    if not kwargs:
-        print('FAIL: Missing dictionary with parameter(s) and value(s)')
+    filename = "/etc/iscsi/iscsid.conf"
+    if not libsan.host.linux.edit_config(filename, parameters, list_values=False):
+        _print(f"FAIL: Unable to set iscsi parameters: {parameters}")
         return False
 
-    for key in list(kwargs.keys()):
-        print(80 * '#')
-        print('INFO: Setting %s to %s in iscsid.conf' % (key, kwargs[key]))
-        print(80 * '#')
-
-        aug.set('/files/etc/iscsi/iscsid.conf/' + key, kwargs[key])
-
-    aug.save()
-
     if not libsan.host.linux.service_restart("iscsid"):
-        _print('FAIL: Unable to restart iscsid service')
+        _print("FAIL: Unable to restart iscsid service")
         return False
 
-    for key in list(kwargs.keys()):
-        value_check = aug.get('/files/etc/iscsi/iscsid.conf/' + key)
-        if value_check == kwargs[key]:
-            print('INFO: %s set successfully to %s' % (key, kwargs[key]))
-
-        else:
-            _print('FAIL: Not able to set iscsid.conf/%s' % key)
-            return False
-
-    aug.close()
-
     return True
 
 
@@ -1055,69 +1000,69 @@ def set_chap(target_user, target_pass, initiator_user=None, initiator_pass=None)
     Arguments:
     target_user, target_pass -- Username and password used for 1-way authentication.
 
-    initiator_user, initiator pass -- optional, used for 2-way bi-directional authentication.
+    initiator_user, initiator pass -- optional, used for 2-way bidirectional authentication.
     """
 
     if not target_user or not target_pass:
-        print('FAIL: set_chap() - requires username and password')
+        print("FAIL: set_chap() - requires username and password")
         return False
 
-    parameters = {'node.session.auth.authmethod': 'CHAP',
-                  'node.session.auth.username': target_user,
-                  'node.session.auth.password': target_pass,
-                  'discovery.sendtargets.auth.authmethod': 'CHAP',  # NetApp array requires discovery authentication
-                  'discovery.sendtargets.auth.username': target_user,
-                  'discovery.sendtargets.auth.password': target_pass}
+    parameters = {
+        "node.session.auth.authmethod": "CHAP",
+        "node.session.auth.username": target_user,
+        "node.session.auth.password": target_pass,
+        "discovery.sendtargets.auth.authmethod": "CHAP",  # NetApp array requires discovery authentication
+        "discovery.sendtargets.auth.username": target_user,
+        "discovery.sendtargets.auth.password": target_pass,
+    }
 
     if initiator_user and initiator_pass:
-        print('INFO: Setting mutual two-way CHAP authentication')
-        parameters['node.session.auth.username_in'] = initiator_user
-        parameters['node.session.auth.password_in'] = initiator_pass
-        parameters['discovery.sendtargets.auth.username_in'] = initiator_user
-        parameters['discovery.sendtargets.auth.password_in'] = initiator_pass
+        print("INFO: Setting mutual two-way CHAP authentication")
+        parameters["node.session.auth.username_in"] = initiator_user
+        parameters["node.session.auth.password_in"] = initiator_pass
+        parameters["discovery.sendtargets.auth.username_in"] = initiator_user
+        parameters["discovery.sendtargets.auth.password_in"] = initiator_pass
 
-    if not set_iscsid_parameter(**parameters):
-        print('FAIL: Unable to set CHAP authentication')
+    if not set_iscsid_parameter(parameters):
+        print("FAIL: Unable to set CHAP authentication")
         return False
 
     if not libsan.host.linux.service_restart("iscsid"):
-        print('FAIL: Unable to restart iscsid service')
+        print("FAIL: Unable to restart iscsid service")
         return False
 
-    print('INFO: CHAP authentication enabled')
+    print("INFO: CHAP authentication enabled")
     return True
 
 
 def disable_chap():
     """Disable CHAP authentication in iscsid.conf and restarts the service"""
 
-    aug = augeas.Augeas()
+    # Removing all previously set auth parameters.
+    parameters = [
+        "node.session.auth.authmethod",
+        "node.session.auth.username",
+        "node.session.auth.password",
+        "discovery.sendtargets.auth.authmethod",
+        "discovery.sendtargets.auth.username",
+        "discovery.sendtargets.auth.password",
+        "node.session.auth.username_in",
+        "node.session.auth.password_in",
+        "discovery.sendtargets.auth.username_in",
+        "discovery.sendtargets.auth.password_in",
+    ]
 
-    # Removing all previously set auth parameters. Commented-out lines stays intact
-    parameters = ['node.session.auth.authmethod',
-                  'node.session.auth.username',
-                  'node.session.auth.password',
-                  'discovery.sendtargets.auth.authmethod',
-                  'discovery.sendtargets.auth.username',
-                  'discovery.sendtargets.auth.password',
-                  'node.session.auth.username_in',
-                  'node.session.auth.password_in',
-                  'discovery.sendtargets.auth.username_in',
-                  'discovery.sendtargets.auth.password_in']
-
-    for param in parameters:
-        aug.remove('/files/etc/iscsi/iscsid.conf/' + param)
-
-    aug.save()
+    libsan.host.linux.remove_from_config("/etc/iscsi/iscsid.conf", parameters, warn=False)
 
     if not libsan.host.linux.service_restart("iscsid"):
-        print('FAIL: Unable to restart iscsid service')
+        print("FAIL: Unable to restart iscsid service")
         return False
 
+    return True
+
 
 def multipath_timeo(seconds=None):
-    """"
-    If multipath is used for iSCSI session, session replacement
+    """If multipath is used for iSCSI session, session replacement
     timeout time should be decreased from default 120 seconds
     https://access.redhat.com/solutions/1171203
     multipathd service should be running when calling this
@@ -1128,32 +1073,26 @@ def multipath_timeo(seconds=None):
     \tFalse: There was some problem.
     """
 
-    param = 'node.session.timeo.replacement_timeout'
+    param = "node.session.timeo.replacement_timeout"
 
     if not seconds:
         seconds = 10
     seconds = str(seconds)
 
     if libsan.host.mp.is_multipathd_running():
-        print('INFO: multipathd is running')
+        print("INFO: multipathd is running")
     else:
-        print('FAIL: multipathd is not running')
+        print("FAIL: multipathd is not running")
         return False
 
-    if not libsan.host.linux.service_restart("iscsid"):
-        _print('FAIL: Unable to restart iscsid service')
-        return False
-
-    if not set_iscsid_parameter(**{param: seconds}):
+    if not set_iscsid_parameter({param: seconds}):
         return False
 
     return True
 
 
 def create_iscsi_iface(iface_name, mac=None):
-    """
-    Create a new iSCSI interface, assign mac if specified
-    """
+    """Create a new iSCSI interface, assign mac if specified"""
     if not iface_name:
         _print("FAIL: create_iscsi_iface() - requires iface name as parameter")
         return False
@@ -1169,15 +1108,14 @@ def create_iscsi_iface(iface_name, mac=None):
         print(output)
         return False
 
-    if mac is not None:
-        if not iface_update(iface_name, 'hwaddress', mac):
-            return False
+    if mac is not None and not iface_update(iface_name, "hwaddress", mac):
+        return False
 
     return True
 
 
 def clone_iscsi_iface(new_iface_name, base_iface):
-    print("Cloning iface: %s to %s" % (base_iface, new_iface_name))
+    print(f"Cloning iface: {base_iface} to {new_iface_name}")
     if not create_iscsi_iface(new_iface_name):
         return False
 
@@ -1186,23 +1124,21 @@ def clone_iscsi_iface(new_iface_name, base_iface):
         print("FAIL: Could not query all info about iface: %s" % base_iface)
         return False
 
-    if iface_info["hwaddress"] is not None:
-        if not iface_update(new_iface_name, "hwaddress", iface_info["hwaddress"]):
-            return False
+    if iface_info["hwaddress"] is not None and not iface_update(new_iface_name, "hwaddress", iface_info["hwaddress"]):
+        return False
 
-    if iface_info["transport_name"] is not None:
+    if iface_info["transport_name"] is not None:  # noqa: SIM102
         if not iface_update(new_iface_name, "transport_name", iface_info["transport_name"]):
             return False
 
-    if iface_info["initiatorname"] is not None:
+    if iface_info["initiatorname"] is not None:  # noqa: SIM102
         if not iface_update(new_iface_name, "initiatorname", iface_info["initiatorname"]):
             return False
 
-    if iface_info["ipaddress"] is not None:
-        if not iface_update(new_iface_name, "ipaddress", iface_info["ipaddress"]):
-            return False
+    if iface_info["ipaddress"] is not None and not iface_update(new_iface_name, "ipaddress", iface_info["ipaddress"]):
+        return False
 
-    print("successully cloned %s. new iface: %s" % (base_iface, new_iface_name))
+    print(f"successully cloned {base_iface}. new iface: {new_iface_name}")
     return True
 
 
@@ -1230,6 +1166,7 @@ def node_iface_info(iface_name):
 
     return True
 
+
 # iSCSI disks ###
 
 
