@@ -1,4 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
 # Copyright (C) 2016 Red Hat, Inc.
 # This file is part of libsan.
 #
@@ -23,8 +22,6 @@ __copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."
 import os.path
 import re
 
-import augeas
-
 import libsan.host.fc
 import libsan.host.iscsi
 import libsan.host.linux
@@ -32,64 +29,70 @@ import libsan.host.lvm
 import libsan.host.net
 import libsan.host.scsi
 import libsan.misc.size
+from libsan import _print
 from libsan.host.cmdline import run
 
-multipath_conf_path = '/etc/multipath.conf'
+multipath_conf_path = "/etc/multipath.conf"
 aug_conf_path = "/files/etc/multipath.conf"
-
-
-def _print(string):
-    module_name = __name__
-    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
-    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
-    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
-    print(string)
-
-    if "FATAL:" in string:
-        raise RuntimeError(string)
-    return
+package_name = "device-mapper-multipath"
 
 
 def mp_service_name():
-    """
-    Return the name of mutlipath service
-    """
+    """Return the name of mutlipath service."""
     return "multipathd"
 
 
 def mp_start_service():
-    """
-    Start multipath service
-    """
+    """Start multipath service."""
+    if libsan.host.linux.is_installed(package_name) and not os.path.isfile(multipath_conf_path):
+        mpathconf_enable()
     return libsan.host.linux.service_start(mp_service_name())
 
 
 def mp_stop_service():
-    """
-    Stop multipath service
-    """
+    """Stop multipath service."""
     return libsan.host.linux.service_stop(mp_service_name())
 
 
 def is_multipathd_running():
-    """
-    Check if multipathd is running
-    """
+    """Check if multipathd is running."""
     return libsan.host.linux.is_service_running(mp_service_name())
 
 
+def mpathconf_enable(find_mpaths=False) -> bool:
+    """Runs 'mpathconf --enable' command"""
+    cmd = "mpathconf --enable"
+    if find_mpaths:
+        cmd = cmd + " --find_multipaths n"
+    retcode = run(cmd)
+    if retcode != 0:
+        return False
+
+    return True
+
+
+def mp_enable() -> bool:
+    """Installs and enable multipath."""
+    if not is_multipathd_running():
+        libsan.host.linux.install_package("device-mapper-multipath")
+        mpathconf_enable()
+        if not libsan.host.linux.service_start(mp_service_name()):
+            return False
+
+    return True
+
+
 def is_mpath_device(mpath_name, print_fail=True):
-    """
-    Checks if given device is multipath.
+    """Checks if given device is multipath.
     :param mpath_name: name of device to check
     :param print_fail: Should we print "FAIL: ..." in case of fail?
-    :return: True / False
+    :return: True / False.
     """
     cmd = "multipath -l %s" % mpath_name
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
         if print_fail:
-            _print("FAIL: Could not execute \"%s\"" % cmd)
+            _print('FAIL: Could not execute "%s"' % cmd)
         return None
 
     # There could be error in the output, so we need to check if we really found the mpath device.
@@ -104,17 +107,17 @@ def get_mpath_of_disk(disk):
     The arguments are:
     \tDisk:   Disk
     Returns:
-    \tString: Return the name of multipath device
+    \tString: Return the name of multipath device.
     """
     if libsan.host.linux.dist_name() == "RHEL" and libsan.host.linux.dist_ver() < 7:
         device = "/dev/%s" % disk
-        mmnum_cmd = "ls -l %s | awk '{print \"(\"$5,$6\")\"}'" % device
+        mmnum_cmd = 'ls -l %s | awk \'{print "("$5,$6")"}\'' % device
         retcode, output = run(mmnum_cmd, return_output=True, verbose=False)
         if retcode != 0:
             _print("FAIL: Could not get device information for %s" % device)
             print(output)
             return None
-        deps_cmd = r"dmsetup deps | grep \"%s\" | awk -v device=%s '{print $1}' | tr -d \"\:\"" % (output, device)
+        deps_cmd = rf"dmsetup deps | grep \"{output}\" | awk -v device={device} '{{print $1}}' | tr -d \"\:\""
         retcode, output = run(deps_cmd, return_output=True, verbose=False)
         if retcode != 0:
             _print("FAIL: Could not get dmsetup information for %s" % device)
@@ -129,8 +132,8 @@ def get_mpath_of_disk(disk):
             _print("FAIL: For some reason multipathd is not running")
             print(output)
             return None
-        fmt = "\"%d %m\""
-        cmd = "multipathd show paths format %s | egrep \"^%s\" | awk '{print$2}'" % (fmt, disk)
+        fmt = '"%d %m"'
+        cmd = f"multipathd show paths format {fmt} | egrep \"^{disk}\" | awk '{{print$2}}'"
         retcode, output = run(cmd, return_output=True, verbose=False)
         if retcode != 0:
             _print("FAIL: to find multipath of disk %s" % disk)
@@ -145,9 +148,7 @@ def get_mpath_of_disk(disk):
 
 
 def get_disks_of_mpath(mpath_name):
-    """
-    Return all SCSI devices that belong to this mpath
-    """
+    """Return all SCSI devices that belong to this mpath."""
     if not mpath_name:
         return None
 
@@ -163,9 +164,8 @@ def get_disks_of_mpath(mpath_name):
 
 
 def get_disks_of_mpath_by_wwpn(mpath_name, wwpn):
-    """
-    From an specific mpath device, return the devices
-    connected to this WWPN
+    """From a specific mpath device, return the devices
+    connected to this WWPN.
     """
     if not mpath_name or not wwpn:
         return None
@@ -176,22 +176,19 @@ def get_disks_of_mpath_by_wwpn(mpath_name, wwpn):
 
     scsi_devices = []
     wwpn = libsan.host.fc.standardize_wwpn(wwpn)
-    if wwpn:
-        if "disk" in list(mpath_dict.keys()):
-            port_entries = ["h_wwpn", "t_wwpn"]
-            for disk in list(mpath_dict["disk"].keys()):
-                for port_entry in port_entries:
-                    if port_entry in list(mpath_dict["disk"][disk].keys()):
-                        if (mpath_dict["disk"][disk][port_entry] and
-                                wwpn in mpath_dict["disk"][disk][port_entry]):
-                            scsi_devices.append(disk)
+    if wwpn and "disk" in list(mpath_dict.keys()):
+        port_entries = ["h_wwpn", "t_wwpn"]
+        for disk in list(mpath_dict["disk"].keys()):
+            for port_entry in port_entries:
+                if port_entry in list(mpath_dict["disk"][disk].keys()):  # noqa: SIM102
+                    if mpath_dict["disk"][disk][port_entry] and wwpn in mpath_dict["disk"][disk][port_entry]:
+                        scsi_devices.append(disk)
     return scsi_devices
 
 
 def get_disks_of_mpath_by_iqn(mpath_name, iqn):
-    """
-    From an specific mpath device, return the devices
-    connected to this IQN
+    """From a specific mpath device, return the devices
+    connected to this IQN.
     """
     if not mpath_name or not iqn:
         return None
@@ -208,17 +205,15 @@ def get_disks_of_mpath_by_iqn(mpath_name, iqn):
         port_entries = ["h_iqn", "t_iqn"]
         for disk in list(mpath_dict["disk"].keys()):
             for port_entry in port_entries:
-                if port_entry in list(mpath_dict["disk"][disk].keys()):
-                    if (mpath_dict["disk"][disk][port_entry] and
-                            iqn in mpath_dict["disk"][disk][port_entry]):
+                if port_entry in list(mpath_dict["disk"][disk].keys()):  # noqa: SIM102
+                    if mpath_dict["disk"][disk][port_entry] and iqn in mpath_dict["disk"][disk][port_entry]:
                         scsi_devices.append(disk)
     return scsi_devices
 
 
 def get_disks_of_mpath_by_mac(mpath_name, mac):
-    """
-    From an specific mpath device, return the devices
-    connected to this MAC
+    """From a specific mpath device, return the devices
+    connected to this MAC.
     """
     if not mpath_name or not mac:
         return None
@@ -235,9 +230,8 @@ def get_disks_of_mpath_by_mac(mpath_name, mac):
         port_entries = ["iface_mac"]
         for disk in list(mpath_dict["disk"].keys()):
             for port_entry in port_entries:
-                if port_entry in list(mpath_dict["disk"][disk].keys()):
-                    if (mpath_dict["disk"][disk][port_entry] and
-                            mac in mpath_dict["disk"][disk][port_entry]):
+                if port_entry in list(mpath_dict["disk"][disk].keys()):  # noqa: SIM102
+                    if mpath_dict["disk"][disk][port_entry] and mac in mpath_dict["disk"][disk][port_entry]:
                         scsi_devices.append(disk)
     return scsi_devices
 
@@ -245,7 +239,7 @@ def get_disks_of_mpath_by_mac(mpath_name, mac):
 def multipath_query_all(mpath_name=None):
     # Not sure with of these 2 commands I should use
     # multipath -ll
-    # will force multipath to issue it's own (synchronous) path checker call,
+    # will force multipath to issue its own (synchronous) path checker call,
     # and report that result.
     # multipathd show top
     # multipath will not run a checker on the paths. It will simply report the
@@ -260,7 +254,7 @@ def multipath_query_all(mpath_name=None):
     # cmd = "multipathd -k\"show top\""
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
-        _print("FAIL: Could not execute \"%s\"" % cmd)
+        _print('FAIL: Could not execute "%s"' % cmd)
         return None
 
     if not output:
@@ -303,10 +297,10 @@ def multipath_query_all(mpath_name=None):
     size_bytes = None
     wwid = None
 
-    all_mpath_dict = dict()
-    all_mpath_dict['by_wwid'] = {}
-    all_mpath_dict['by_mpath_name'] = {}
-    all_mpath_dict['by_scsi_id'] = {}
+    all_mpath_dict = {}
+    all_mpath_dict["by_wwid"] = {}
+    all_mpath_dict["by_mpath_name"] = {}
+    all_mpath_dict["by_scsi_id"] = {}
     mpath_dict = {}
 
     scsi_host_id_2_driver = {}
@@ -410,7 +404,7 @@ def multipath_query_all(mpath_name=None):
             match_scsi_id = re.search(libsan.host.scsi.get_regex_scsi_id(), line)
             if match_scsi_id:
                 scsi_host_id = match_scsi_id.group(1)
-                fc_target_id = "%s:%s:%s" % (match_scsi_id.group(1), match_scsi_id.group(2), match_scsi_id.group(3))
+                fc_target_id = f"{match_scsi_id.group(1)}:{match_scsi_id.group(2)}:{match_scsi_id.group(3)}"
                 if scsi_host_id in list(scsi_host_id_2_driver.keys()):
                     scsi_driver = scsi_host_id_2_driver[scsi_host_id]
 
@@ -430,7 +424,7 @@ def multipath_query_all(mpath_name=None):
                         target_ip = iscsi_session["target_ip"]
                         persist_ip = iscsi_session["persist_ip"]
 
-            disk_info_dict = dict()
+            disk_info_dict = {}
             disk_info_dict["scsi_id"] = scsi_id
             disk_info_dict["host_id"] = host_id
             disk_info_dict["scsi_disk"] = scsi_disk
@@ -460,7 +454,8 @@ def multipath_query_all(mpath_name=None):
 
             if h_wwpn:
                 disk_info_dict["transport_type"] = libsan.host.fc.fc_host_transport_type(
-                    libsan.host.fc.fc_host_id_of_wwpn(h_wwpn))
+                    libsan.host.fc.fc_host_id_of_wwpn(h_wwpn)
+                )
 
             if scsi_driver == "scsi_debug":
                 disk_info_dict["transport_type"] = "scsi_debug"
@@ -469,53 +464,43 @@ def multipath_query_all(mpath_name=None):
             if disk_info_dict["transport_type"] not in mpath_dict["transport_types"]:
                 mpath_dict["transport_types"].append(disk_info_dict["transport_type"])
 
-            if scsi_driver:
-                if scsi_driver not in mpath_dict["scsi_drivers"]:
-                    mpath_dict["scsi_drivers"].append(scsi_driver)
+            if scsi_driver and scsi_driver not in mpath_dict["scsi_drivers"]:
+                mpath_dict["scsi_drivers"].append(scsi_driver)
 
-            if host_id:
-                if host_id not in mpath_dict["host_ids"]:
-                    mpath_dict["host_ids"].append(host_id)
+            if host_id and host_id not in mpath_dict["host_ids"]:
+                mpath_dict["host_ids"].append(host_id)
 
-            if t_wwpn:
-                if t_wwpn not in mpath_dict["t_wwpns"]:
-                    mpath_dict["t_wwpns"].append(t_wwpn)
+            if t_wwpn and t_wwpn not in mpath_dict["t_wwpns"]:
+                mpath_dict["t_wwpns"].append(t_wwpn)
 
-            if h_wwpn:
-                if h_wwpn not in mpath_dict["h_wwpns"]:
-                    mpath_dict["h_wwpns"].append(h_wwpn)
+            if h_wwpn and h_wwpn not in mpath_dict["h_wwpns"]:
+                mpath_dict["h_wwpns"].append(h_wwpn)
 
             if h_wwpn and t_wwpn:
                 map_info = {"t_wwpn": t_wwpn, "h_wwpn": h_wwpn}
                 mpath_dict["map_info"].append(map_info)
 
-            if t_iqn:
-                if t_iqn not in mpath_dict["t_iqns"]:
-                    mpath_dict["t_iqns"].append(t_iqn)
+            if t_iqn and t_iqn not in mpath_dict["t_iqns"]:
+                mpath_dict["t_iqns"].append(t_iqn)
 
-            if h_iqn:
-                if h_iqn not in mpath_dict["h_iqns"]:
-                    mpath_dict["h_iqns"].append(h_iqn)
+            if h_iqn and h_iqn not in mpath_dict["h_iqns"]:
+                mpath_dict["h_iqns"].append(h_iqn)
 
             if h_iqn and t_iqn:
                 map_info = {"t_iqn": t_iqn, "h_iqn": h_iqn}
                 mpath_dict["map_info"].append(map_info)
 
-            if iface_mac:
-                if iface_mac not in mpath_dict["iface_macs"]:
-                    mpath_dict["iface_macs"].append(iface_mac)
+            if iface_mac and iface_mac not in mpath_dict["iface_macs"]:
+                mpath_dict["iface_macs"].append(iface_mac)
 
-            if iface_name:
-                if h_iqn not in mpath_dict["iface_names"]:
-                    mpath_dict["iface_names"].append(iface_name)
+            if iface_name and h_iqn not in mpath_dict["iface_names"]:
+                mpath_dict["iface_names"].append(iface_name)
 
-            if target_ip:
-                if target_ip not in mpath_dict["target_ips"]:
-                    mpath_dict["target_ips"].append(target_ip)
+            if target_ip and target_ip not in mpath_dict["target_ips"]:
+                mpath_dict["target_ips"].append(target_ip)
 
-            if persist_ip:
-                if persist_ip not in mpath_dict["persist_ips"]:
-                    mpath_dict["persist_ips"].append(persist_ip)
+            if persist_ip and persist_ip not in mpath_dict["persist_ips"]:
+                mpath_dict["persist_ips"].append(persist_ip)
 
             mpath_dict["disk"][disk_info_dict["scsi_disk"]] = disk_info_dict
             mpath_dict["path_group"][pg_id]["disk"][scsi_id] = disk_info_dict
@@ -525,7 +510,7 @@ def multipath_query_all(mpath_name=None):
 
             continue
 
-        # as regex_mpath is a agressive regex, we leave it at the last
+        # as regex_mpath is an agressive regex, we leave it at the last
         # one to check.
         m = re.match(regex_mpath, line)
         if m:
@@ -554,7 +539,7 @@ def multipath_query_all(mpath_name=None):
                 product_raw = m.group(5)
                 product = re.sub("[ ]+$", "", product_raw)
 
-            mpath_dict = dict()
+            mpath_dict = {}
             mpath_dict["mpath_name"] = _mpath_name
             mpath_dict["wwid"] = wwid
             mpath_dict["dm_name"] = dm_name
@@ -582,8 +567,6 @@ def multipath_query_all(mpath_name=None):
 
 
 def mpath_name_of_wwid(wwid):
-    """
-    """
     if not wwid:
         _print("FAIL: mpath_name_of_wwid() - requires wwid parameter")
         return None
@@ -599,9 +582,7 @@ def mpath_name_of_wwid(wwid):
 
 
 def mpath_names_of_vendor(vendor):
-    """
-    Given a Vendor return a list with all multipath device names with this vendor
-    """
+    """Given a Vendor return a list with all multipath device names with this vendor."""
     if not vendor:
         _print("FAIL: mpath_names_of_vendor() - requires vendor parameter")
         return None
@@ -619,9 +600,7 @@ def mpath_names_of_vendor(vendor):
 
 
 def mpath_check_disk_status(mpath_name, scsi_disk):
-    """
-    Check the online status of an specific SCSI disk from a mpath device
-    """
+    """Check the online status of a specific SCSI disk from a mpath device."""
     if not mpath_name or not scsi_disk:
         _print("FAIL: mpath_check_disk_status() - requires mpath_name and scsi_disk parameters")
         return None
@@ -643,9 +622,7 @@ def mpath_check_disk_status(mpath_name, scsi_disk):
 
 
 def mpath_check_disk_dm_status(mpath_name, scsi_disk):
-    """
-    Check the dm status of an specific SCSI disk from a mpath device
-    """
+    """Check the dm status of a specific SCSI disk from a mpath device."""
     if not mpath_name or not scsi_disk:
         _print("FAIL: mpath_check_disk_dm_status() - requires mpath_name and scsi_disk parameters")
         return None
@@ -667,9 +644,7 @@ def mpath_check_disk_dm_status(mpath_name, scsi_disk):
 
 
 def mpath_check_disk_path_status(mpath_name, scsi_disk):
-    """
-    Check the path status of an specific SCSI disk from a mpath device
-    """
+    """Check the path status of a specific SCSI disk from a mpath device."""
     if not mpath_name or not scsi_disk:
         _print("FAIL: mpath_check_disk_path_status() - requires mpath_name and scsi_disk parameters")
         return None
@@ -691,10 +666,9 @@ def mpath_check_disk_path_status(mpath_name, scsi_disk):
 
 
 def mpath_get_active_disk(mpath_name):
-    """
-    From specific mpath device get which disk is the active one
+    """From specific mpath device get which disk is the active one
     Return
-    \tList of active SCSI disks
+    \tList of active SCSI disks.
     """
     if not mpath_name:
         _print("FAIL: mpath_get_active_disk() - requires mpath_name parameter")
@@ -723,9 +697,7 @@ def mpath_get_active_disk(mpath_name):
 
 
 def get_free_mpaths(exclude_boot_device=True, exclude_lvm_device=True):
-    """
-    Return a dict of free mpath devices
-    """
+    """Return a dict of free mpath devices."""
     all_mp_info = multipath_query_all()
     if not all_mp_info:
         # could not query multipath devices
@@ -739,7 +711,7 @@ def get_free_mpaths(exclude_boot_device=True, exclude_lvm_device=True):
     pvs = libsan.host.lvm.pv_query()
     boot_dev = libsan.host.linux.get_boot_device()
     boot_wwid = None
-    # if for some reason we bot from a single disk, but this disk is part of multipath device
+    # if for some reason we boot from a single disk, but this disk is part of multipath device
     # the mpath device should be skipped as well
     if boot_dev:
         boot_wwid = libsan.host.linux.get_device_wwid(boot_dev)
@@ -770,9 +742,7 @@ def get_free_mpaths(exclude_boot_device=True, exclude_lvm_device=True):
 
 
 def h_wwpns_of_mpath(mpath_name):
-    """
-    Return the h_wwpns of specific mpath device
-    """
+    """Return the h_wwpns of specific mpath device."""
     if not mpath_name:
         _print("FAIL: h_wwpns_of_mpath() - requires mpath_name parameter")
         return None
@@ -788,9 +758,7 @@ def h_wwpns_of_mpath(mpath_name):
 
 
 def t_wwpns_of_mpath(mpath_name):
-    """
-    Return the h_wwpns of specific mpath device
-    """
+    """Return the h_wwpns of specific mpath device."""
     if not mpath_name:
         _print("FAIL: t_wwpns_of_mpath() - requires mpath_name parameter")
         return None
@@ -806,9 +774,7 @@ def t_wwpns_of_mpath(mpath_name):
 
 
 def iface_macs_of_mpath(mpath_name):
-    """
-    Return the iface_macs_of_mpath of specific mpath device
-    """
+    """Return the iface_macs_of_mpath of specific mpath device."""
     if not mpath_name:
         _print("FAIL: iface_macs_of_mpath() - requires mpath_name parameter")
         return None
@@ -824,9 +790,7 @@ def iface_macs_of_mpath(mpath_name):
 
 
 def transport_types_of_mpath(mpath_name):
-    """
-    Return the transport_types of specific mpath device
-    """
+    """Return the transport_types of specific mpath device."""
     if not mpath_name:
         _print("FAIL: transport_types_of_mpath() - requires mpath_name parameter")
         return None
@@ -849,8 +813,7 @@ def multipath_show(mpath_name=None):
 
 
 def multipath_reload(mpath_name=None):
-    """
-    Usage
+    """Usage
     multipath_reload()
     Purpose
         Execute 'multipath -r'
@@ -861,7 +824,7 @@ def multipath_reload(mpath_name=None):
             or
         undef
     Exceptions
-        N/A
+        N/A.
     """
     cmd = "multipath -r"
     if mpath_name:
@@ -874,9 +837,7 @@ def multipath_reload(mpath_name=None):
 
 
 def remove_mpath(mpath_name):
-    """
-    Remove specific mpath
-    """
+    """Remove specific mpath."""
     if not mpath_name:
         _print("FAIL: remove_mpath() - requires mpath_name parameter")
         return False
@@ -888,9 +849,7 @@ def remove_mpath(mpath_name):
 
 
 def flush_all():
-    """
-    Flush all unused multipath device maps
-    """
+    """Flush all unused multipath device maps."""
     cmd = "multipath -F"
     ret = run(cmd, verbose=True)
     if ret != 0:
@@ -900,8 +859,7 @@ def flush_all():
 
 
 def multipath_backup_conf(bak_file):
-    """
-    backup_mp_conf ()
+    """backup_mp_conf ()
     Usage
     backup_mp_conf(bak_file)
     Purpose
@@ -913,23 +871,22 @@ def multipath_backup_conf(bak_file):
         or
     False           # file exists or source file not exists;
     Exceptions
-    N/A
+    N/A.
     """
-    mpath_conf = '/etc/multipath.conf'
 
-    if not os.path.isfile(mpath_conf):
-        _print("FAIL: %s does not exist" % mpath_conf)
+    if not os.path.isfile(multipath_conf_path):
+        _print("FAIL: %s does not exist" % multipath_conf_path)
         return False
 
     if os.path.isfile(bak_file):
         _print("FAIL: mpath backup file %s already exists" % bak_file)
         return False
 
-    _print("INFO: Backing up %s to %s" % (mpath_conf, bak_file))
-    cmd = "cp -f %s %s" % (mpath_conf, bak_file)
+    _print(f"INFO: Backing up {multipath_conf_path} to {bak_file}")
+    cmd = f"cp -f {multipath_conf_path} {bak_file}"
     ret, output = run(cmd, return_output=True)
     if ret != 0:
-        _print("FAIL: Could not backup %s" % mpath_conf)
+        _print("FAIL: Could not backup %s" % multipath_conf_path)
         print(output)
         return False
 
@@ -937,8 +894,7 @@ def multipath_backup_conf(bak_file):
 
 
 def multipath_restore_conf(bak_file):
-    """
-    multipath_restore_conf ()
+    """multipath_restore_conf ()
     Usage
     multipath_restore_conf(bak_file)
     Purpose
@@ -951,14 +907,14 @@ def multipath_restore_conf(bak_file):
         or
     False           # file exists or source file not exists;
     Exceptions
-    N/A
+    N/A.
     """
     if not os.path.isfile(bak_file):
         _print("FAIL: %s does not exist" % bak_file)
         return False
 
     _print("INFO: Restoring multipath configuration from %s" % bak_file)
-    cmd = "cp -f %s %s" % (bak_file, multipath_conf_path)
+    cmd = f"cp -f {bak_file} {multipath_conf_path}"
     ret, output = run(cmd, return_output=True)
     if ret != 0:
         _print("FAIL: Could not restore backup from %s" % bak_file)
@@ -970,8 +926,7 @@ def multipath_restore_conf(bak_file):
 
 
 def multipath_reload_conf():
-    """
-    Usage
+    """Usage
     multipath_reload_conf()
     Purpose
         Execute "multiapthd -k'reconfig'" to load configuration.
@@ -983,7 +938,7 @@ def multipath_reload_conf():
             or
         undef
     Exceptions
-        N/A
+        N/A.
     """
     cmd = "multipathd -k'reconfig'"
     ret = run(cmd, verbose=True)
@@ -996,14 +951,13 @@ def multipath_reload_conf():
 
 
 def multipath_setup_blacklist():
-    """
-    multipath_setup_blacklist ()
+    """multipath_setup_blacklist ()
     Usage
     multipath_setup_blacklist()
     Purpose
     Add 'wwid .*' into blacklist and all exists mpath wwid to
     'blacklist_exceptions'. This will prevent mpath take over newly
-    created LUN. This function DO NOT backup configuration, use
+    created LUN. This function DO NOT back up configuration, use
     backup_mp_conf() in stead.
     Parameter
     N/A
@@ -1012,7 +966,7 @@ def multipath_setup_blacklist():
         or
     undef
     Exceptions
-    N/A
+    N/A.
     """
     all_mp_info_dict = multipath_query_all()
     if not all_mp_info_dict:
@@ -1021,11 +975,11 @@ def multipath_setup_blacklist():
     if not current_wwids:
         return False
 
-    if not mpath_conf_add('/blacklist/wwid', ".*"):
+    if not mpath_conf_set("/blacklist/wwid", ".*"):
         return False
 
     for wwid in current_wwids:
-        if not mpath_conf_add('/blacklist_exceptions/wwid', wwid):
+        if not mpath_conf_set("/blacklist_exceptions/wwid[last()+1]", wwid):
             return False
 
     _print("INFO: Reloading updated multipath configuration")
@@ -1033,98 +987,84 @@ def multipath_setup_blacklist():
     return True
 
 
-def mpath_conf_add(path, value):
-    """
-    Checks if the specified path/value is present in config, and adds it if not.
-    Use this if the path is not unique - eg. multiple wwid's can be listed.
-    For valid options use 'man multipath.conf'
-
-    :param path: eg. "/blacklist/wwid"
-    :param value: eg. "<device wwid>"
-    :returns: boolean
-    :raises IOError: Augeas save fails. Check if multipath.conf changes are valid
-    """
-
-    aug = augeas.Augeas()
-    full_path = aug_conf_path + path
-
-    matched = aug.match(full_path + "[. = '%s']" % value)
-
-    if not matched:
-        try:
-            aug.set(full_path + '[last() + 1]', value)
-            aug.save()
-            aug.close()
-        except IOError:
-            _print('FAIL: Unable to add %s to %s, Check if multipath.conf changes are valid' % (value, path))
-            return False
-        except Exception as e:
-            print("Exception: %s" % e)
-            return False
-
-    return True
-
-
 def mpath_conf_remove(path, value):
-    """
-    Remove parameter from multipath config.
-    For valid options use 'man multipath.conf'
+    """Remove parameter from multipath config using augeas.
+    For non-unique paths, use return of mptah_conf_match as path
+    For valid options use 'man multipath.conf'.
 
     :param path: eg. "/blacklist/wwid"
     :param value: eg. "<device wwid>"
     :returns: boolean
     :raises IOError: Augeas save fails. Check if multipath.conf changes are valid.
     """
+    matched_path = mpath_conf_match(path, value)
 
-    aug = augeas.Augeas()
-    full_path = aug_conf_path + path
-
-    try:
-        aug.remove(full_path + "[. = '%s']" % value)
-        aug.save()
-        aug.close()
-    except IOError:
-        _print('FAIL: Unable to remove %s / %s, Check if multipath.conf changes are valid' % (path, value))
-        return False
-    except Exception as e:
-        print("Exception: %s" % e)
-        return False
+    if matched_path:
+        _, out = run(f'augtool -s rm "{matched_path}"', return_output=True, verbose=False)
+        if "Saved 1 file(s)" not in out:
+            _print(f"FAIL: unable to set {path} to {value}")
+            return False
 
     return True
 
 
-def mpath_conf_set(path, value):
-    """
-    Change multipath.conf parameters.
-    For valid options use 'man multipath.conf'
+def mpath_conf_match(path, value):
+    """Checks if parameter is set in multipath.conf using augeas
+    Use path expressions to get non-unique paths: https://github.com/hercules-team/augeas/wiki/Path-expressions
+    e.g. path="/devices/device[*]/vendor" value="LIO-ORG"
+    Successfully matches even if "end of the path" is not unique. e.g. path="/blacklist_exceptions/wwid" value="$wwid".
 
-    :param path: eg. "/defaults/path_selector"
+    :param path: eg. "/blacklist/wwid"
+    :param value: eg. "<device wwid>"
+    :returns: matched path or None
+    """
+    if not libsan.host.linux.install_package("augeas", verbose=False):
+        _print("FAIL: Could not install augeas")
+        return None
+
+    full_path = path if path.startswith(aug_conf_path) else aug_conf_path + path
+
+    # successfully matches also when end of path is not unique - path[*] value, but not path[*]/path value
+    _, out = run(f'augtool match "{full_path}" "{value}"', return_output=True, verbose=False)
+
+    # augtool prints path when matched, return codes seems useless
+    if aug_conf_path in out:
+        # returning path, as it can be useful when modifying non-unique paths
+        return out
+    return None
+
+
+def mpath_conf_set(path, value):
+    """Change multipath.conf parameters using augeas.
+    To append to non-unique paths, use [last()+1].
+    eg. path="/devices/device[last()+1]/vendor" value="LIO-ORG"
+    For valid options use 'man multipath.conf'.
+
+    :param path: eg. "/defaults/path_selector" or "/files/etc/multipath.conf/defaults/path_selector"
     :param value: eg. "round-robin"
     :returns: boolean
     :raises IOError: Augeas save fails. Check if multipath.conf changes are valid.
     """
+    full_path = path if path.startswith(aug_conf_path) else aug_conf_path + path
 
-    aug = augeas.Augeas()
-    full_path = aug_conf_path + path
+    # Need to check if not already exists to avoid duplicates
+    path_to_match = re.sub(r"\[[^]]*]", "[.]", path)  # in case path expressions are being used
+    matched_path = mpath_conf_match(path_to_match, value)
 
-    try:
-        aug.set(full_path, value)
-        aug.save()
-        aug.close()
-    except IOError:
-        _print('FAIL: multipath.conf - unable to set %s / %s' % (path, value))
-        return False
-    except Exception as e:
-        print("Exception: %s" % e)
-        return False
+    if not matched_path:
+        if not libsan.host.linux.is_installed("augeas"):
+            _print("FAIL: Could not install augeas")
+            return False
+        _, out = run(f'augtool -s set "{full_path}" "{value}"', return_output=True, verbose=False)
+        if "Saved 1 file(s)" not in out:
+            _print(f"FAIL: unable to set {path} to {value}. Try")
+            return False
 
     return True
 
 
 def mp_query_conf(config_str):
-    """
-    Parse string containing multipath config to a dict
-    """
+    """Parse string containing multipath config to a dict."""
     regex_option = r"^[ \t]*"
     regex_option += r"([^\ #=\t]+)"
     regex_option += r"[\ \t]+"
@@ -1150,7 +1090,7 @@ def mp_query_conf(config_str):
                     continue
                 if "devs" not in list(config_dict[section_name].keys()):
                     config_dict[section_name]["devs"] = []
-                # We are in a sub section, need to update current_section
+                # We are in a subsection, need to update current_section
                 tmp_dict = {}
                 config_dict[section_name]["devs"].append(tmp_dict)
                 # poiting current_section to last item in the list
@@ -1162,7 +1102,7 @@ def mp_query_conf(config_str):
                     continue
                 if "mpaths" not in list(config_dict[section_name].keys()):
                     config_dict[section_name]["mpaths"] = []
-                # We are in a sub section, need to update current_section
+                # We are in a subssection, need to update current_section
                 tmp_dict = {}
                 config_dict[section_name]["mpaths"].append(tmp_dict)
                 # poiting current_section to last item in the list
@@ -1200,15 +1140,14 @@ def mp_query_conf(config_str):
 
 
 def mp_query_saved_conf():
-    """
-    Usage
+    """Usage
         mp_query_saved_conf()
     Purpose
         Load multipath config file and return it as a dict
     Parameter
         N/A
     Returns
-        mp_conf_dict
+        mp_conf_dict.
     """
     if not os.path.isfile(multipath_conf_path):
         _print("FAIL: %s does not exist" % multipath_conf_path)
@@ -1222,24 +1161,18 @@ def mp_query_saved_conf():
 
 
 def _load_config(config_file):
-    """
-    Parse multipath config file to dict
-    """
-    mpath_cfg_file = open(config_file, encoding='UTF-8')
-    mpath_cfg = mpath_cfg_file.read()
-
-    return mpath_cfg
+    """Parse multipath config file to dict."""
+    with open(config_file) as f:
+        return f.read()
 
 
 def _save_config(config_dict, config_file=multipath_conf_path):
-    """
-    Convert multipath config dict to string and save to file
-    """
+    """Convert multipath config dict to string and save to file."""
     config_str = ""
     if "defaults" in list(config_dict.keys()):
         config_str += "defaults {\n"
         for key in list(config_dict["defaults"].keys()):
-            config_str += "\t%s %s\n" % (key, config_dict["defaults"][key])
+            config_str += "\t{} {}\n".format(key, config_dict["defaults"][key])
         config_str += "}\n"
 
     blacklist_sections = ["blacklist", "blacklist_exceptions"]
@@ -1249,17 +1182,17 @@ def _save_config(config_dict, config_file=multipath_conf_path):
 
             if "devnodes" in list(config_dict[b_section].keys()):
                 for node in config_dict[b_section]["devnodes"]:
-                    config_str += "\tdevnode \"%s\"\n" % node
+                    config_str += '\tdevnode "%s"\n' % node
 
             if "wwids" in list(config_dict[b_section].keys()):
                 for wwid in config_dict[b_section]["wwids"]:
-                    config_str += "\twwid \"%s\"\n" % wwid
+                    config_str += '\twwid "%s"\n' % wwid
 
             if "devs" in list(config_dict[b_section].keys()):
                 for device in config_dict[b_section]["devs"]:
                     config_str += "\tdevice {\n"
                     for key in list(device.keys()):
-                        config_str += "\t\t%s %s\n" % (key, device[key])
+                        config_str += f"\t\t{key} {device[key]}\n"
                     config_str += "\t}\n"
 
             config_str += "}\n"
@@ -1270,7 +1203,7 @@ def _save_config(config_dict, config_file=multipath_conf_path):
             for product in config_dict["devices"][vendor]:
                 config_str += "\tdevice {\n"
                 for name in list(product.keys()):
-                    config_str += "\t\t%s %s\n" % (name, product[name])
+                    config_str += f"\t\t{name} {product[name]}\n"
                 config_str += "\t}\n"
         config_str += "}\n"
 
@@ -1279,21 +1212,19 @@ def _save_config(config_dict, config_file=multipath_conf_path):
         for key in list(config_dict["multipaths"].keys()):
             config_str += "\tmultipath {\n"
             for wwid in config_dict["multipaths"][key]:
-                config_str += "\t\t%s %s\n" % (key, wwid)
+                config_str += f"\t\t{key} {wwid}\n"
             config_str += "}\n"
         config_str += "}\n"
 
-    f = open(config_file, "w", encoding='UTF-8')
-    f.write(config_str)
-    f.close()
+    with open(config_file, "w") as f:
+        f.write(config_str)
 
     return True
 
 
 def mpath_device_2_mpath_name(mpath_device):
-    """
-    Convert an specific multipath device to mpath_name
-    Eg. /dev/mapper/mpathap1 => mpatha
+    """Convert a specific multipath device to mpath_name
+    E.g. /dev/mapper/mpathap1 => mpatha.
     """
     multipath_dev_regex = r"/dev/mapper\/(.*)"
     m = re.match(multipath_dev_regex, mpath_device)
@@ -1311,7 +1242,7 @@ def mpath_device_2_mpath_name(mpath_device):
             # does not seem to be a valid mpath device
             return None
         device_name = m.group(1)
-        # remove trailing p if it exist
+        # remove trailing p if it exists
         device_name = re.sub(r"(\S+)p$", r"\1", device_name)
         return device_name
     return None
