@@ -1,5 +1,3 @@
-from __future__ import absolute_import, division, print_function, unicode_literals
-
 # Copyright (C) 2016 Red Hat, Inc.
 # This file is part of libsan.
 #
@@ -23,14 +21,17 @@ __copyright__ = "Copyright (c) 2016 Red Hat, Inc. All rights reserved."
 
 import os.path
 import re  # regex
-import libsan.host.mp
+from os import listdir, readlink
+
 import libsan.host.linux
 import libsan.host.lvm
+import libsan.host.md
+import libsan.host.mp
 import libsan.host.net
 import libsan.misc.array
 import libsan.misc.size
-from libsan.host.cmdline import run
-from os import listdir, readlink
+from libsan import _print
+from libsan.host.cmdline import exists, run
 
 # I'm still note sure whether to use /sys/class/scsi_disk or /sys/class/scsi_device
 
@@ -38,32 +39,16 @@ sys_disk_path = "/sys/class/scsi_disk"
 host_path = "/sys/class/scsi_host"
 
 # add /lib/udev to PATH because scsi_id is located there on RHEL7
-os.environ['PATH'] += ":/lib/udev"
+os.environ["PATH"] += ":/lib/udev"
 
 
 def get_regex_scsi_id():
     return "([0-9]+):([0-9]+):([0-9]+):([0-9]+)"
 
 
-def _print(string):
-    module_name = __name__
-    string = re.sub("DEBUG:", "DEBUG:(" + module_name + ") ", string)
-    string = re.sub("FAIL:", "FAIL:(" + module_name + ") ", string)
-    string = re.sub("FATAL:", "FATAL:(" + module_name + ") ", string)
-    string = re.sub("WARN:", "WARN:(" + module_name + ") ", string)
-    print(string)
-    if "FATAL:" in string:
-        raise RuntimeError(string)
-    return
-
-
 def is_scsi_device(scsi_device):
-    """
-    """
-    if re.match("^sd[a-z]+$", scsi_device):
-        return True
-    else:
-        return False
+    """ """
+    return bool(re.match("^sd[a-z]+$", scsi_device))
 
 
 def get_scsi_disk_ids():
@@ -88,7 +73,7 @@ def get_scsi_disk_name(device_id):
         _print("FAIL: get_scsi_disk_name() requires scsi_device_id as parameter")
         return None
 
-    cmd = "ls %s/%s/device/block" % (sys_disk_path, device_id)
+    cmd = f"ls {sys_disk_path}/{device_id}/device/block"
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
         return None
@@ -101,7 +86,7 @@ def get_scsi_disk_vendor(device_id):
         _print("FAIL: get_scsi_disk_vendor() requires scsi_device_id as parameter")
         return None
 
-    cmd = "cat %s/%s/device/vendor" % (sys_disk_path, device_id)
+    cmd = f"cat {sys_disk_path}/{device_id}/device/vendor"
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
         return None
@@ -114,7 +99,7 @@ def get_scsi_disk_model(device_id):
         _print("FAIL: get_scsi_disk_model() requires scsi_device_id as parameter")
         return None
 
-    cmd = "cat %s/%s/device/model" % (sys_disk_path, device_id)
+    cmd = f"cat {sys_disk_path}/{device_id}/device/model"
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
         return None
@@ -123,8 +108,7 @@ def get_scsi_disk_model(device_id):
 
 
 def query_all_scsi_disks(scsi_disk=None):
-    """
-    Query information of all SCSI disks and return them as a dict
+    """Query information of all SCSI disks and return them as a dict
     SCSI ID is the dict key.
     If an SCSI disk is given as argument, return its info
     Parameter:
@@ -159,19 +143,21 @@ def query_all_scsi_disks(scsi_disk=None):
         if m:
             host_id = m.group(1)
             driver = scsi_driver_of_host_id(host_id)
-        scsi_info = {"name": scsi_name,
-                     "wwid": scsi_wwid,
-                     "wwn": scsi_wwn,  # Uses scsi_id to query WWN
-                     "udev_wwn": udev_wwn,  # Used udevadm to query WWN
-                     "size": size_bytes,
-                     "size_human": libsan.misc.size.size_bytes_2_size_human(size_bytes),
-                     "state": state,
-                     "timeout": timeout,
-                     "vendor": scsi_vendor,
-                     "model": scsi_model,
-                     "host_id": host_id,
-                     "driver": driver,
-                     "scsi_id": disk_id}
+        scsi_info = {
+            "name": scsi_name,
+            "wwid": scsi_wwid,
+            "wwn": scsi_wwn,  # Uses scsi_id to query WWN
+            "udev_wwn": udev_wwn,  # Used udevadm to query WWN
+            "size": size_bytes,
+            "size_human": libsan.misc.size.size_bytes_2_size_human(size_bytes),
+            "state": state,
+            "timeout": timeout,
+            "vendor": scsi_vendor,
+            "model": scsi_model,
+            "host_id": host_id,
+            "driver": driver,
+            "scsi_id": disk_id,
+        }
         scsi_disks[disk_id] = scsi_info
 
     if scsi_disk:
@@ -184,8 +170,7 @@ def query_all_scsi_disks(scsi_disk=None):
 
 
 def get_scsi_name_by_vendor(vendor):
-    """
-    Query information of all SCSI disks and return all scsi device names that
+    """Query information of all SCSI disks and return all scsi device names that
     are from the requested vendor
     Parameter:
     \tvendor:        SCSI disk Vendor. eg: 'LIO'
@@ -201,15 +186,13 @@ def get_scsi_name_by_vendor(vendor):
         return None
     scsi_names = []
     for scsi_info in list(all_scsi_disks_info.values()):
-        if "vendor" in list(scsi_info.keys()):
-            if scsi_info["vendor"] == vendor:
-                scsi_names.append(scsi_info["name"])
+        if "vendor" in list(scsi_info.keys()) and scsi_info["vendor"] == vendor:
+            scsi_names.append(scsi_info["name"])
     return scsi_names
 
 
 def scsi_host_of_scsi_name(scsi_name):
-    """
-    """
+    """ """
     if not scsi_name:
         _print("FAIL: scsi_host_of_scsi_name() - requires scsi_name parameter")
         return None
@@ -222,8 +205,7 @@ def scsi_host_of_scsi_name(scsi_name):
 
 
 def scsi_name_2_scsi_id(scsi_name):
-    """
-    """
+    """ """
     if not scsi_name:
         _print("FAIL: scsi_name_2_scsi_id() - requires scsi_name parameter")
         return None
@@ -236,9 +218,7 @@ def scsi_name_2_scsi_id(scsi_name):
 
 
 def delete_disk(device_name):
-    """
-    device_name:    eg. sda
-    """
+    """device_name:    eg. sda"""
     if not device_name:
         _print("FAIL: delete_disk() requires scsi_device_name as parameter")
         return None
@@ -248,7 +228,7 @@ def delete_disk(device_name):
         _print("FAIL: delete_disk() could not find disk %s" % device_name)
         return None
 
-    cmd = "echo \"1\" >  %s/%s/device/delete" % (sys_disk_path, device_id)
+    cmd = f'echo "1" >  {sys_disk_path}/{device_id}/device/delete'
     retcode = run(cmd, verbose=False)
     if retcode != 0:
         return None
@@ -256,8 +236,7 @@ def delete_disk(device_name):
 
 
 def get_hosts(somethings=None):
-    """
-    Return a list with all SCSI hosts.
+    """Return a list with all SCSI hosts.
     The arguments are:
     \tNone
     or
@@ -281,7 +260,7 @@ def get_hosts(somethings=None):
 
     scsi_host_ids = None
     # If somethings is a single string, convert it to list
-    for something in somethings if not isinstance(somethings, type("")) else [somethings]:
+    for something in somethings if not isinstance(somethings, str) else [somethings]:
         if something is None:
             _print("FAIL: get_hosts() - Invalid input")
             print(somethings)
@@ -309,8 +288,7 @@ def get_hosts(somethings=None):
 
 
 def query_scsi_host_info(host_id):
-    """
-    Usage
+    """Usage
         query_scsi_host_info(scsi_host_id)
     Purpose
         Save sysfs info of "/sys/class/scsi_host/host$scsi_host_id" to
@@ -334,14 +312,14 @@ def query_scsi_host_info(host_id):
         _print("FAIL: %s is not a valid directory" % host_id)
         return None
 
-    scsi_host_info = dict()
+    scsi_host_info = {}
     scsi_host_info["scsi_host_id"] = host_id
     scsi_host_info["pci_id"] = pci_id_of_host_id(host_id)
     scsi_host_info["driver"] = scsi_driver_of_host_id(host_id)
 
     param_files = [f for f in listdir(sysfs_folder) if os.path.isfile(os.path.join(sysfs_folder, f))]
     for param in param_files:
-        ret, output = run("cat %s/%s" % (sysfs_folder, param), return_output=True, verbose=False)
+        ret, output = run(f"cat {sysfs_folder}/{param}", return_output=True, verbose=False)
         if ret != 0:
             # For some reason could not read the file
             continue
@@ -353,7 +331,7 @@ def query_scsi_host_info(host_id):
             continue
         host_files = [x for x in listdir(sysfs_host) if os.path.isfile(os.path.join(sysfs_host, x))]
         for param in host_files:
-            ret, output = run("cat %s/%s" % (sysfs_host, param), return_output=True, verbose=False)
+            ret, output = run(f"cat {sysfs_host}/{param}", return_output=True, verbose=False)
             if ret != 0:
                 # For some reason could not read the file
                 continue
@@ -363,8 +341,7 @@ def query_scsi_host_info(host_id):
 
 
 def scsi_driver_of_host_id(host_id):
-    """
-    """
+    """ """
     if not host_id:
         _print("FAIL: scsi_driver_of_host_id() - requires host_id parameter")
         return None
@@ -401,10 +378,10 @@ def scsi_driver_of_host_id(host_id):
         pci_id = pci_id_of_host_id(host_id)
         if pci_id:
             lspci_regex = r"Kernel modules:\s+(\S+)"
-            if run("which lspci", verbose=False) != 0:
+            if not exists("lspci"):
                 _print("FAIL: pciutils is not installed. Can't query driver name using pci_id.")
                 return None
-            _, output = run("lspci -s \"%s\" -v | grep \"Kernel modules:\"" % pci_id, return_output=True, verbose=False)
+            _, output = run('lspci -s "%s" -v | grep "Kernel modules:"' % pci_id, return_output=True, verbose=False)
             if output:
                 m = re.search(lspci_regex, output)
                 if m:
@@ -423,8 +400,7 @@ def scsi_driver_of_host_id(host_id):
 
 
 def pci_id_of_host_id(host_id):
-    """
-    Usage
+    """Usage
         pci_id_of(scsi_host_id);
     Purpose
         Find out which PCI ID providing the SCSI Host via:
@@ -445,7 +421,7 @@ def pci_id_of_host_id(host_id):
     link_path = readlink(sys_path)
 
     regex_pci_id = libsan.host.linux.get_regex_pci_id()
-    m = re.search("(%s)/host%s/scsi_host" % (regex_pci_id, host_id), link_path)
+    m = re.search(f"({regex_pci_id})/host{host_id}/scsi_host", link_path)
     # _print("DEBUG: pci_id_of_host_id - %s" % link_path)
     if m:
         return m.group(1)
@@ -461,8 +437,7 @@ def pci_id_of_host_id(host_id):
 
 
 def rescan_host(host=None, verbose=True):
-    """
-    Rescan for devices for specific host
+    """Rescan for devices for specific host
     If no host is given it will scan all SCSI hosts
     The arguments are:
     \tHost:      eg. 1 for host1
@@ -487,7 +462,7 @@ def rescan_host(host=None, verbose=True):
     for host in host_list:
         if verbose:
             _print("INFO: Rescanning host%s" % host)
-        cmd = "echo \"- - -\" > %s/host%s/scan" % (host_path, host)
+        cmd = f'echo "- - -" > {host_path}/host{host}/scan'
         retcode, _ = run(cmd, return_output=True, verbose=verbose)
         if retcode != 0:
             error += 1
@@ -500,8 +475,7 @@ def rescan_host(host=None, verbose=True):
 
 
 def rescan_disk(scsi_disk=None):
-    """
-    Rescan an specific SCSI disk.
+    """Rescan an specific SCSI disk.
     If no disk is given, rescann all SCSI disks
     echo 1 > /sys/block/<scsi_disk>/device/rescan
     \tHost:      eg. 1 for host1
@@ -547,20 +521,15 @@ def size_of_disk(scsi_disk):
     if not scsi_disk:
         return None
 
-    cmd = "cat /sys/block/%s/queue/logical_block_size" % scsi_disk
-    retcode, output = run(cmd, return_output=True, verbose=False)
-    if retcode != 0:
-        _print("FAIL: size_of_disk() - Could not get size for disk %s" % scsi_disk)
-        print(output)
+    logical_block_size = get_logical_block_size_of_disk(scsi_disk)
+
+    if not logical_block_size:
         return None
-    if not output:
-        return None
-    logical_block_size = output
 
     cmd = "cat /sys/block/%s/size" % scsi_disk
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
-        _print("FAIL: size_of_disk() - Could not get sectore size for disk %s" % scsi_disk)
+        _print("FAIL: size_of_disk() - Could not get sector size for disk %s" % scsi_disk)
         print(output)
         return None
     if not output:
@@ -571,9 +540,32 @@ def size_of_disk(scsi_disk):
     return int(logical_block_size) * int(sector_size)
 
 
+def get_logical_block_size_of_disk(scsi_disk):
+    cmd = "cat /sys/block/%s/queue/logical_block_size" % scsi_disk
+    retcode, output = run(cmd, return_output=True, verbose=False)
+    if retcode != 0:
+        _print("FAIL: get_logical_block_size_of_disk() - Could not get logical block size for disk %s" % scsi_disk)
+        print(output)
+        return None
+    if not output:
+        return None
+    return output
+
+
+def get_physical_block_size_of_disk(scsi_disk):
+    cmd = "cat /sys/block/%s/queue/physical_block_size" % scsi_disk
+    retcode, output = run(cmd, return_output=True, verbose=False)
+    if retcode != 0:
+        _print("FAIL: get_physical_block_size_of_disk() - Could not get size for disk %s" % scsi_disk)
+        print(output)
+        return None
+    if not output:
+        return None
+    return output
+
+
 def wwid_of_disk(scsi_disk=None):
-    """
-    Usage
+    """Usage
         wwid_of_disk(disk)
     Purpose
         Given an scsi_disk name. Eg. sda, mpatha
@@ -587,14 +579,13 @@ def wwid_of_disk(scsi_disk=None):
         return None
 
     if libsan.host.linux.is_dm_device(scsi_disk):
-        return libsan.host.linux.get_udev_property(scsi_disk, 'DM_SERIAL')
+        return libsan.host.linux.get_udev_property(scsi_disk, "DM_SERIAL")
 
-    return libsan.host.linux.get_udev_property(scsi_disk, 'ID_SERIAL')
+    return libsan.host.linux.get_udev_property(scsi_disk, "ID_SERIAL")
 
 
 def scsi_ids_of_wwid(wwid):
-    """
-    Usage
+    """Usage
         scsi_ids_of_wwid(wwid)
     Purpose
         Find out all SCSI ID for WWID.
@@ -625,8 +616,7 @@ def scsi_ids_of_wwid(wwid):
 
 
 def wwn_of_disk(scsi_disk):
-    """
-    Usage
+    """Usage
         wwn_of_disk(disk)
     Purpose
         Given an scsi_disk name. Eg. sda
@@ -682,8 +672,7 @@ def wwn_of_disk(scsi_disk):
 
 
 def udev_wwn_of_disk(scsi_disk):
-    """
-    Usage
+    """Usage
         udev_wwn_of_disk(disk)
     Purpose
         Given an scsi_disk name. Eg. sda
@@ -715,8 +704,7 @@ def udev_wwn_of_disk(scsi_disk):
 
 
 def query_scsi_driver_info(driver):
-    """
-    """
+    """ """
     if not driver:
         _print("FAIL: query_scsi_driver_info() - requires driver parameter")
         return None
@@ -736,7 +724,7 @@ def query_scsi_driver_info(driver):
         _print("FAIL:  No SCSI disk found from driver %s" % driver)
         return None
 
-    scsi_driver_info = dict()
+    scsi_driver_info = {}
     scsi_driver_info["scsi_host"] = {}
     for host_id in driver_host_ids:
         scsi_driver_info["scsi_host"][host_id] = query_scsi_host_info(host_id)
@@ -748,9 +736,14 @@ def query_scsi_driver_info(driver):
     return scsi_driver_info
 
 
-def get_free_disks(exclude_boot_device=True, exclude_lvm_device=True, exclude_mpath_device=True, filter_only=None):
-    """
-    Return a dict of free SCSI devices.
+def get_free_disks(
+    exclude_boot_device=True,
+    exclude_lvm_device=True,
+    exclude_mpath_device=True,
+    exclude_md_device=True,
+    filter_only=None,
+):
+    """Return a dict of free SCSI devices.
     By default it excludes devices used for boot, lvm or multipath
     Optional "filter_only" argument should be a dict. Eg. filter_only={'state': 'running'}
     """
@@ -760,6 +753,7 @@ def get_free_disks(exclude_boot_device=True, exclude_lvm_device=True, exclude_mp
         return None
 
     pvs = libsan.host.lvm.pv_query()
+    md_devices = libsan.host.md.md_query()
     boot_dev = libsan.host.linux.get_boot_device()
     boot_wwid = None
     # if for some reason we boot from a single disk, but this disk is part of multipath device
@@ -786,13 +780,34 @@ def get_free_disks(exclude_boot_device=True, exclude_lvm_device=True, exclude_mp
         if all_mp_info and scsi_info["wwid"] in list(all_mp_info["by_wwid"].keys()) and exclude_mpath_device:
             _print("DEBUG: get_free_disks() - skip %s as it is used for mpath" % scsi_info["name"])
             continue
+
+        # Skip if it is used by Soft RAID
+        if md_devices and exclude_md_device:
+            used_by_md = False
+            for md_dev in md_devices:
+                storage_devs = libsan.host.md.md_get_storage_dev(md_dev)
+                if not storage_devs:
+                    continue
+                for dev in storage_devs:
+                    dev_wwid = wwid_of_disk(dev)
+                    if not dev_wwid:
+                        continue
+                    if dev_wwid == scsi_info["wwid"]:
+                        _print("DEBUG: get_free_disks() - skip %s as it is used for md" % scsi_info["name"])
+                        used_by_md = True
+                        continue
+            if used_by_md:
+                continue
+
         # Skip if filter_only is specified
         filtered = False
         if filter_only is not None:
             for key in filter_only:
                 if scsi_info[key] != filter_only[key]:
-                    _print("DEBUG: get_free_disks() - filtered %s as %s is not %s" % (
-                        scsi_info["name"], key, filter_only[key]))
+                    _print(
+                        "DEBUG: get_free_disks() - filtered %s as %s is not %s"
+                        % (scsi_info["name"], key, filter_only[key])
+                    )
                     filtered = True
                     continue
         if filtered:
@@ -803,7 +818,7 @@ def get_free_disks(exclude_boot_device=True, exclude_lvm_device=True, exclude_mp
         # Skip if it is used by LVM
         if pvs and exclude_lvm_device:
             for pv in list(pvs.keys()):
-                if '/' + get_scsi_disk_name(scsi_disk) in pv:
+                if "/" + get_scsi_disk_name(scsi_disk) in pv:
                     _print("DEBUG: get_free_disks() - skip %s as it is used for LVM" % scsi_info["name"])
                     chosen_disks.pop(scsi_info["name"])
 
@@ -811,8 +826,7 @@ def get_free_disks(exclude_boot_device=True, exclude_lvm_device=True, exclude_mp
 
 
 def scsi_device_2_scsi_name(scsi_device):
-    """
-    Convert an specific SCSI device to scsi_name
+    """Convert an specific SCSI device to scsi_name
     Eg. /dev/sdap1 => sda
     """
     scsi_dev_regex = r"/dev\/(sd.*)"
@@ -829,8 +843,7 @@ def scsi_device_2_scsi_name(scsi_device):
 
 
 def disk_sys_trigger(scsi_disk, action):
-    """
-    Usage
+    """Usage
         disk_sys_trigger(scsi_disk, action)
     Purpose
         Bring disk online/offline, via
@@ -859,7 +872,7 @@ def disk_sys_trigger(scsi_disk, action):
     if action == "DOWN":
         action = "offline"
 
-    cmd = "echo %s > %s" % (action, sys_path)
+    cmd = f"echo {action} > {sys_path}"
     retcode, output = run(cmd, return_output=True, verbose=False)
     if retcode != 0:
         _print("FAIL: disk_sys_trigger() - Could not execute %s" % cmd)
@@ -873,15 +886,14 @@ def disk_sys_trigger(scsi_disk, action):
         return False
 
     if action != new_state:
-        _print("FAIL: disk_sys_trigger() - Current state is '%s' expected '%s'" % (new_state, action))
+        _print(f"FAIL: disk_sys_trigger() - Current state is '{new_state}' expected '{action}'")
         return False
 
     return True
 
 
 def disk_sys_check(scsi_disk):
-    """
-    Usage
+    """Usage
         disk_sys_check(scsi_disk)
     Purpose
         Check state of specific disk
@@ -912,8 +924,7 @@ def disk_sys_check(scsi_disk):
 
 
 def timeout_of_disk(scsi_disk):
-    """
-    Usage
+    """Usage
         timeout_of_disk(scsi_disk)
     Purpose
         Check timeout of specific disk
